from re import search, sub

from bs4 import BeautifulSoup as bs
from requests import get
from requests.exceptions import ConnectTimeout, ReadTimeout


class LimeTorrents:
    def __init__(self, user_agent):
        self.url = "https://limetorrents.info"
        self.search = "/search/all/##"
        self.delimiter = "-"
        self.sort = "/search/all/##/@@"
        self.sort_type = {
            "age": "date",
            "size": "size",
            "seed": "seeds",
            "leech": "leechs",
        }
        self.page = "/@@"
        self.user_agent = {"User-Agent": user_agent}

    def _get(self, url):
        return get(
            url, allow_redirects=True, headers=self.user_agent, timeout=3, verify=False
        )

    def _search_torrents(self, query, pages, sort=None):
        endpoint = (
            sub(
                "@@",
                self.sort_type[sort],
                sub("##", sub(r"\s", self.delimiter, query), self.sort),
            )
            if sort and sort in self.sort_type
            else sub("##", sub(r"\s", self.delimiter, query), self.search)
        )
        search_texts = list()
        for i in range(pages):
            page_url = sub("@@", str(i), self.page)
            soup = bs(self._get(f"{self.url}{endpoint}{page_url}").text, "html.parser")
            search_texts.extend(soup.findAll("td")[12:])
        return search_texts

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except (ConnectTimeout, ReadTimeout):
            return []
        singles = [search_list[i * 6 : i * 6 + 6] for i in range(len(search_list) // 6)]
        torrents = list()
        for torrent in singles:
            name = torrent[0].text
            _, link_page = [t.get("href") for t in torrent[0].findAll("a")]
            info, size, seed, leech = [t.text for t in torrent[1:5]]
            last_check, category = info.split(" - in ")
            torrents.append(
                [
                    "LimeTorrents",
                    name,
                    f"{self.url}{link_page}",
                    last_check,
                    category,
                    size,
                    int(sub(",", "", seed)),
                    int(sub(",", "", leech)),
                ]
            )
        return torrents

    def get_magnet_link(self, torrent_page):
        try:
            soup = bs(self._get(torrent_page).text, "html.parser")
            return [
                x.get("href")
                for x in soup.findAll("a", {"class": "csprite_dltorrent"})
                if search("magnet", x.get("href"))
            ][0]
        except IndexError:
            return None



from datetime import datetime, timedelta
from re import search, sub

from bs4 import BeautifulSoup as bs
from requests import get
from requests.exceptions import ConnectTimeout, ReadTimeout


class TorLock:
    def __init__(self, user_agent):
        self.url = "https://www.torlock.com"
        self.search = "/all/torrents/##"
        self.delimiter = "+"
        self.sort = "/all/torrents/##.html?sort=@@&order=desc"
        self.sort_type = {
            "age": "added",
            "size": "size",
            "seed": "seeds",
            "leech": "peers",
        }
        self.page = "&page=@@"
        self.user_agent = {"User-Agent": user_agent}

    def _get(self, url):
        return get(
            url, allow_redirects=True, headers=self.user_agent, timeout=3, verify=False
        )

    def _search_torrents(self, query, pages, sort=None):
        endpoint = (
            sub(
                "@@",
                self.sort_type[sort],
                sub("##", sub(r"\s", self.delimiter, query), self.sort),
            )
            if sort and sort in self.sort_type
            else sub("##", sub(r"\s", self.delimiter, query), self.search)
        )
        search_texts = list()
        for i in range(pages):
            page_url = sub("@@", str(i + 1), self.page)
            soup = bs(self._get(f"{self.url}{endpoint}{page_url}").text, "html.parser")
            search_texts.extend(soup.findAll("td")[27:])
        return search_texts

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except (ConnectTimeout, ReadTimeout):
            return []
        singles = [search_list[i * 7 : i * 7 + 7] for i in range(len(search_list) // 7)]
        torrents = list()
        for torrent in singles:
            name = torrent[0].text
            link_page = [a.get("href") for a in torrent[0].findAll("a")]
            if link_page:
                category = int(torrent[0].find("span").get("class")[0][2:])
                time, size, seed, leech = [t.text for t in torrent[1:5]]
                torrents.append(
                    [
                        "TorLock",
                        name,
                        f"{self.url}{link_page[0]}",
                        self._date_converter(time),
                        self._get_category(category),
                        sub(r"\d+$", "", size),
                        int(sub(",", "", seed)),
                        int(sub(",", "", leech)),
                    ]
                )
        return torrents

    def _date_converter(self, site_date):
        if site_date == "Today":
            return datetime.now().strftime("%d.%m.%Y")
        if site_date == "Yesterday":
            return datetime.strftime(datetime.now() - timedelta(1), "%d.%m.%Y")
        day, month, year = site_date.split("/")
        return "{:02d}.{:02d}.{}".format(int(month), int(day), int(year))

    def _get_category(self, category_id):
        categories = {
            1: "Movies",
            2: "Music",
            3: "Television",
            4: "Games",
            5: "Software",
            6: "Anime",
            7: "Adult",
            8: "E-Books",
            9: "Images",
            12: "Audio Books",
            0: "Other",
        }
        return categories[category_id] if category_id in categories else "Other"

    def get_magnet_link(self, torrent_page):
        try:
            soup = bs(self._get(torrent_page).text, "html.parser")
            return [
                a.get("href")
                for a in soup.findAll("a")
                if search("magnet", str(a.get("href")))
            ][0]
        except IndexError:
            return None



from datetime import datetime
from re import search, sub

import requests
from humanfriendly import format_size
from requests.exceptions import ConnectTimeout, ReadTimeout


class ThePirateBay:
    def __init__(self, user_agent):
        self.url = "https://thepiratebay.org"
        self.api_url = "https://apibay.org"
        self.search = "/q.php?q=##"
        self.delimiter = "%20"
        self.page = "/description.php?id=@@"
        self.user_agent = {"User-Agent": user_agent}

    def _search_torrents(self, query, pages, sort=None):
        url_attach = sub("##", sub(r"\s", self.delimiter, query), self.search)
        return requests.get(f"{self.api_url}{url_attach}").json()

    def _parse_date(self, timestamp):
        if not timestamp or not timestamp.isdigit():
            return 0
        return format(datetime.fromtimestamp(int(timestamp)), "%d.%m.%Y")

    def _get_category(self, category_id):
        major_category_id = int(category_id[0])
        categories = {
            1: "Audio",
            2: "Video",
            3: "App",
            4: "Games",
            5: "Porn",
            6: "Other",
        }
        return categories.get(major_category_id, "Other")

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except (ConnectTimeout, ReadTimeout):
            return []
        if search_list[0].get("name") == "No results returned":
            return []
        return [
            [
                "ThePirateBay",
                torrent.get("name"),
                f"{self.url}{sub('@@',torrent.get('id'),self.page)}",
                self._parse_date(torrent.get("added", 0)),
                self._get_category(torrent.get("category")),
                format_size(int(torrent.get("size"), 0)),
                int(torrent.get("seeders")),
                int(torrent.get("leechers")),
            ]
            for torrent in search_list
        ]

    def get_magnet_link(self, torrent_page):
        torrent_id = search(r"id\=(\d+)", torrent_page).group(1)
        info_url = f"{self.api_url}/t.php?id={torrent_id}"
        try:
            info_torrent = requests.get(info_url).json()
        except Exception:
            return None
        torrent_name = info_torrent.get("name").strip().replace(" ", ".")
        torrent_hash = info_torrent.get("info_hash")
        return f"magnet:?xt=urn:btih:{torrent_hash}&dn={torrent_name}"



from re import search, sub

from bs4 import BeautifulSoup as bs
from requests import get
from requests.exceptions import ConnectTimeout, ReadTimeout


class CorsaroNero:
    def __init__(self, user_agent):
        self.url = "https://ilcorsaronero.link"
        self.search = "/search?q=##"
        self.delimiter = "+"
        self.sort = "/search?q=##&sort=@@&order=desc"
        self.sort_type = {
            "age": "timestamp",
            "size": "size",
            "seed": "seeders",
            "leech": "leechers",
        }
        self.page = "&page=@@"
        self.user_agent = {"User-Agent": user_agent}

    def _get(self, url):
        return get(
            url, allow_redirects=True, headers=self.user_agent, timeout=3, verify=False
        )

    def _search_torrents(self, query, pages, sort=None):
        endpoint = (
            sub(
                "@@",
                self.sort_type[sort],
                sub("##", sub(r"\s", self.delimiter, query), self.sort),
            )
            if sort and sort in self.sort_type
            else sub("##", sub(r"\s", self.delimiter, query), self.search)
        )
        titles_list = list()
        search_texts = list()
        for i in range(pages):
            page_url = sub("@@", str(i), self.page)
            soup = bs(self._get(f"{self.url}{endpoint}{page_url}").text, "html.parser")
            search_texts.extend(soup.findAll("td"))
            titles_list.extend(soup.findAll("th"))
        return search_texts, titles_list[7:]

    def build_list(self, query, pages, sort=None):
        try:
            search_list, title_list = self._search_torrents(query, pages, sort)
        except (ConnectTimeout, ReadTimeout):
            return []
        search_list = [x for x in search_list if x.text and not x.get("colspan")]
        singles = [search_list[i * 6 : i * 6 + 6] for i in range(len(search_list) // 6)]
        torrents = list()
        for torrent, title in zip(singles, title_list):
            category = torrent[0].text
            torrent_page = title.find("a").get("href")
            seed, leech, size, time = [
                "0" if x == "n/a" else x for x in [t.text for t in torrent[1:5]]
            ]
            torrents.append(
                [
                    "CorsaroNero",
                    title.text,
                    f"{self.url}{torrent_page}",
                    time.strip().split(" ")[0],
                    category.strip(),
                    sub(r"\d+$", "", size.strip()),
                    int(sub(",", "", seed)),
                    int(sub(",", "", leech)),
                ]
            )
        return torrents

    def get_magnet_link(self, torrent_page):
        try:
            soup = bs(self._get(torrent_page).text, "html.parser")
            return [
                a.get("href")
                for a in soup.findAll("a")
                if search("magnet", str(a.get("href")))
            ][0]
        except IndexError:
            return None
